@@ -1,8 +1,8 @@
 /*
  * tnef.c -- extract files from microsoft TNEF format
  *
- * Copyright (C)1999, 2000, 2001 Mark Simpson <damned@world.std.com>
- * Copyright (C)1998 Thomas Boll  <tb@boll.ch>		[ORIGINAL AUTHOR]
+ * Copyright (C)1999-2003 Mark Simpson <damned@world.std.com>
+ * Copyright (C)1998 Thomas Boll  <tb@boll.ch>	[ORIGINAL AUTHOR]
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,35 +30,34 @@
 #include <assert.h>
 #include <errno.h>
 #include <stdio.h>
-#include <stdlib.h>
-#include <sys/stat.h>
-#include <unistd.h>
+
+#if HAVE_SYS_STAT_H
+#  include <sys/stat.h>
+#endif
 
 #if STDC_HEADERS
-#  if HAVE_STRING_H
-#    include <string.h>
-#  else
-#    include <strings.h>
-#  endif
+#  include <stdarg.h>
+#  include <stdlib.h>
+#  include <string.h>
+#  include <memory.h>
 #else
-#  if HAVE_MEMMOVE
-#    define memmove(d, s, n) bcopy ((s), (d), (n))
-#  endif/* HAVE_MEMMOVE */
-#endif /* STDC_HEADERS */
+extern int strcmp (const char *, const char *);
+#  if !HAVE_MEMMOVE
+#    define memmove(d,s,n) bcopy((s),(d),(n))
+#  else
+extern void* memmove (void *, const void *, size_t);
+#  endif
+#endif
 
-#include <stdarg.h>
-
-#include "strdup.h"
+#include "basename.h"
 #include "ldiv.h"
+#include "strdup.h"
 
 #include "tnef.h"
 
 #include "alloc.h"
 #include "path.h"
 
-static char ident_string[] =
-"$Id: tnef.c,v 1.28 2001/08/14 01:42:28 simpson Exp $";
-
 /* To quiet compiler define tempnam */
 extern char*
 tempnam(const char*, const char*);
@@ -71,10 +70,11 @@ tempnam(const char*, const char*);
 #define GETINT16(p)    (uint16)((uint8)(p)[0]+((uint8)(p)[1]<<8))
 
 /* Global variables, used by all (or nearly all) functions */
-static int8 g_flags = NONE;/* program options */
+static int8 g_flags = NONE;     /* program options */
 static char *g_directory = NULL; /* output directory */
 static FILE *g_file = NULL;     /* input file */
 
+
 /* Array of days of the week for translating a date */
 static char* day_of_week[] = { "Sun", "Mon", "Tue",
                                "Wed", "Thu", "Fri", "Sat" }; 
@@ -107,6 +107,29 @@ typedef struct
     char* buf;
 } Attr;
 
+typedef struct
+{
+    unsigned long data1;
+    unsigned short data2;
+    unsigned short data3;
+    unsigned char data4[8];
+} MAPI_GUID;
+
+typedef struct
+{
+    unsigned short id;
+    unsigned short chbgtrp;
+    unsigned short cch;
+    unsigned short cb;
+} TRP;
+
+typedef struct
+{
+    TRP trp;
+    char* sender_display_name;
+    char* sender_address;
+} TRIPLE;
+
 typedef struct
 {
     size_t len;
@@ -116,6 +139,7 @@ typedef struct
         uint16 bytes2;
         uint32 bytes4;
         uint32 bytes8[2];
+        MAPI_GUID guid;
     } data;
 } MAPI_Value;
 
@@ -191,10 +215,10 @@ debug_print (const char *fmt, ...)
 static char *
 find_free_number (const char *fname)
 {
-    char *tmp = MALLOC (strlen (fname) 
-                        + 1 /* '.' */
-                        + 5 /* big enough for our purposes (i hope) */
-                        + 1 /* NULL */);
+    char *tmp = CHECKED_MALLOC (strlen (fname) 
+                                + 1 /* '.' */
+                                + 5 /* big enough for our purposes (i hope) */
+                                + 1 /* NULL */);
     int counter = 1;
     struct stat buf;
 
@@ -366,19 +390,36 @@ copy_date_from_attr (Attr* attr, struct date* dt)
     {
         memmove (dt, attr->buf, attr->len);
         dt->year = GETINT16((unsigned char*)&dt->year);
-	    dt->month = GETINT16((unsigned char*)&dt->month);
-	    dt->day = GETINT16((unsigned char*)&dt->day);
-	    dt->hour = GETINT16((unsigned char*)&dt->hour);
-	    dt->min = GETINT16((unsigned char*)&dt->min);
-	    dt->sec = GETINT16((unsigned char*)&dt->sec);
-	    dt->dow = GETINT16((unsigned char*)&dt->dow);
-	} 
+        dt->month = GETINT16((unsigned char*)&dt->month);
+        dt->day = GETINT16((unsigned char*)&dt->day);
+        dt->hour = GETINT16((unsigned char*)&dt->hour);
+        dt->min = GETINT16((unsigned char*)&dt->min);
+        dt->sec = GETINT16((unsigned char*)&dt->sec);
+        dt->dow = GETINT16((unsigned char*)&dt->dow);
+    } 
     else 
     {
-	    fprintf (stderr, "date attribute in %s failed sanity check\n",
-                 get_tnef_name_str (attr->name));
-	    memset (dt, 0, sizeof(*dt));
-	}
+        char *name = get_tnef_name_str (attr->name);
+        fprintf (stderr, "date attribute in %s failed sanity check\n",
+                 name);
+        FREE(name);
+        memset (dt, 0, sizeof(*dt));
+    }
+}
+
+void
+copy_triple_from_attr (Attr* attr, TRIPLE *t)
+{
+    assert (attr);
+    assert (t);
+    assert (attr->type == szTRIPLES);
+    
+    t->trp.id = GETINT16 (attr->buf);
+    t->trp.chbgtrp = GETINT16 (attr->buf+2);
+    t->trp.cch = GETINT16 (attr->buf+4);
+    t->trp.cb = GETINT16 (attr->buf+6);
+    t->sender_display_name = attr->buf+8;
+    t->sender_address = attr->buf+8+t->trp.cch;
 }
 
 /* dump_attr
@@ -392,9 +433,13 @@ dump_attr (Attr* attr)
     struct date dt;
     uint16 s;
     uint32 l;
-    unsigned long i;
+    size_t i;
 
-    fprintf (stdout, "%s [type: %s] =", name, type);
+    fprintf (stdout, "%s [type: %s] [len: %lu] =",
+             name, type, (unsigned long)attr->len);
+
+    FREE(name);
+    FREE(type);
 
     switch (attr->type)
     {
@@ -412,7 +457,7 @@ dump_attr (Attr* attr)
             fprintf (stdout, "Not enough data for szSHORT");
             abort();
         }
-		s = GETINT16(attr->buf);
+        s = GETINT16(attr->buf);
         fprintf (stdout, " " SHORT_INT_FMT, s);
         if (attr->len > sizeof(uint16))
         {
@@ -431,7 +476,7 @@ dump_attr (Attr* attr)
             fprintf (stdout, "Not enough data for szLONG");
             abort();
         }
-		l = GETINT32(attr->buf);
+        l = GETINT32(attr->buf);
         fprintf (stdout, " " LONG_INT_FMT, l);
         if (attr->len > sizeof(uint32))
         {
@@ -472,7 +517,7 @@ dump_attr (Attr* attr)
     case szTEXT:
     case szSTRING:
         {
-            char* buf = MALLOC ((attr->len + 1) * sizeof (char));
+            char* buf = CHECKED_MALLOC ((attr->len + 1) * sizeof (char));
             strncpy (buf, (char*)attr->buf, attr->len);
             buf[attr->len] = '\0';
             fprintf (stdout, "'%s'", buf);
@@ -480,9 +525,26 @@ dump_attr (Attr* attr)
         }
         break;
 
+    case szTRIPLES:
+        {
+            TRIPLE triple;
+            copy_triple_from_attr (attr, &triple);
+            fprintf (stdout, 
+                     "{id=%d,chgtrp=%d,cch=%d,cb=%d} "
+                     "sender_display_name='%s', "
+                     "sender_address='%s'",
+                     triple.trp.id,
+                     triple.trp.chbgtrp,
+                     triple.trp.cch,
+                     triple.trp.cb,
+                     triple.sender_display_name,
+                     triple.sender_address);
+        }
+        break;
+
     default:
-        fprintf (stdout, "<unknown type> len=%lu", 
-                 (unsigned long)attr->len);
+        fprintf (stdout, "<unknown type>");
+        break;
     }
     fprintf (stdout, "\n");
 }
@@ -492,17 +554,21 @@ dump_attr (Attr* attr)
 static void
 dump_mapi_attr (MAPI_Attr* attr)
 {
-    const char *name = get_mapi_name_str (attr->name);
-    const char *type = get_mapi_type_str (attr->type);
-    int i;
+    char *name = get_mapi_name_str (attr->name);
+    char *type = get_mapi_type_str (attr->type);
+    size_t i;
 
-    fprintf (stdout, "%s [type: %s] [num_values = %d] = \n", 
-             name, type, attr->num_values);
+    fprintf (stdout, "%s [type: %s] [num_values = %lu] = \n", 
+             name, type, (unsigned long)attr->num_values);
+
+    FREE(name);
+    FREE(type);
 
     for (i = 0; i < attr->num_values; i++)
     {
-        fprintf (stdout, "\tvalue #%d [len: %d] = ", i, 
-                 attr->values[i].len);
+        fprintf (stdout, "\t#%lu [len: %lu] = ", 
+                 (unsigned long)i, 
+                 (unsigned long)attr->values[i].len);
 
         switch (attr->type)
         {
@@ -529,6 +595,7 @@ dump_mapi_attr (MAPI_Attr* attr)
         case szMAPI_BOOLEAN:
             fprintf (stdout, "%s",
                      ((attr->values[i].data.bytes2 == 0) ? "false" : "true"));
+            break;
 
         case szMAPI_STRING:
         case szMAPI_UNICODE_STRING:
@@ -536,25 +603,42 @@ dump_mapi_attr (MAPI_Attr* attr)
                      attr->values[i].data.buf);
             break;
 
+        case szMAPI_SYSTIME:
         case szMAPI_CURRENCY:
         case szMAPI_INT8BYTE:
-        case szMAPI_SYSTIME:
+        case szMAPI_APPTIME:
             fprintf (stdout, "%x %x",
                      (int)attr->values[i].data.bytes8[0],
                      (int)attr->values[i].data.bytes8[1]);
             break;
 
-        case szMAPI_APPTIME:
         case szMAPI_ERROR:
-        case szMAPI_OBJECT:
-        case szMAPI_CLSID:
             fprintf (stdout, "%x",
                      attr->values[i].data.bytes4);
             break;
 
+        case szMAPI_CLSID:
+            {
+                int j;
+                fprintf (stdout, "{%04lx %02x %02x ",
+                         attr->values[i].data.guid.data1,
+                         attr->values[i].data.guid.data2,
+                         attr->values[i].data.guid.data3);
+                fprintf (stdout, "{");
+                for (j = 0; i < 8; i++)
+                {
+                    fprintf (stdout, "%x",
+                             attr->values[i].data.guid.data4[j]);
+                }
+                fprintf (stdout, "}");
+            }
+            break;
+
+        case szMAPI_OBJECT:
         case szMAPI_BINARY:
             {
                 int x;
+                
                 for (x = 0; x < attr->values[i].len; x++)
                 {
                     if (x < 10) 
@@ -585,7 +669,7 @@ dump_mapi_attr (MAPI_Attr* attr)
 static int
 check_checksum (Attr* attr, uint16 checksum)
 {
-    uint32 i;
+    size_t i;
     uint16 sum = 0;
 
     for (i = 0; i < attr->len; i++)
@@ -605,6 +689,7 @@ decode_object (void)
     char buf[2];
     size_t bytes_read = 0;
     uint16 checksum = 0;
+
     /* First we must get the lvl type */
     if (fread (buf, 1, 1, g_file) == 0) 
     { 
@@ -620,20 +705,20 @@ decode_object (void)
     }
     else
     {
-        Attr *attr = (Attr*)MALLOC (sizeof(Attr));
-        memset (attr, '\0', sizeof (Attr));
+        Attr *attr = (Attr*)CHECKED_CALLOC (1, sizeof(Attr));
         
         attr->lvl_type = (uint8)buf[0];
+
         assert ((attr->lvl_type == LVL_MESSAGE) 
                 || (attr->lvl_type == LVL_ATTACHMENT));
         
         type_and_name = geti32();
-        
+ 
         attr->type = (type_and_name >> 16);
         attr->name = ((type_and_name << 16) >> 16);
         attr->len = geti32();
-        attr->buf = MALLOC (attr->len);
-        
+        attr->buf = CHECKED_CALLOC (attr->len, sizeof(char));
+
         bytes_read = fread (attr->buf, 1, attr->len, g_file);
         if (bytes_read != attr->len)
         {
@@ -681,11 +766,8 @@ alloc_mapi_values (MAPI_Attr* a)
 {
     if (a && a->num_values)
     {
-        a->values = (MAPI_Value*)MALLOC (a->num_values * 
-                                         sizeof (MAPI_Value));
-        memset (a->values, 
-                '\0', 
-                (a->num_values * sizeof (MAPI_Value)));
+        a->values = (MAPI_Value*)CHECKED_CALLOC (a->num_values,
+                                                 sizeof (MAPI_Value));
         return a->values;
     }
     return NULL;
@@ -699,8 +781,8 @@ decode_mapi (size_t len, char *buf)
     int i;
     uint32 num_properties = GETINT32(buf+idx);
     MAPI_Attr** attrs= 
-        (MAPI_Attr**)MALLOC ((num_properties + 1) * 
-                             sizeof (MAPI_Attr*));
+        (MAPI_Attr**)CHECKED_MALLOC ((num_properties + 1) * 
+                                     sizeof (MAPI_Attr*));
 
     idx += 4;
 
@@ -708,51 +790,67 @@ decode_mapi (size_t len, char *buf)
     for (i = 0; i < num_properties; i++)
     {
         MAPI_Attr* a = attrs[i] = 
-            (MAPI_Attr*)MALLOC(sizeof (MAPI_Attr));
+            (MAPI_Attr*)CHECKED_CALLOC(1, sizeof (MAPI_Attr));
         MAPI_Value* v = NULL;
-        
-        memset (a, '\0', sizeof (MAPI_Attr));
-        
+
         a->type = GETINT16(buf+idx); idx += 2;
         a->name = GETINT16(buf+idx); idx += 2;
+
         switch (a->type)
         {
         case szMAPI_SHORT:        /* 2 bytes */
             a->num_values = 1;
             v = alloc_mapi_values (a);
             v->len = 2;
-            memmove (&v->data, buf+idx, 2); 
-            idx += 2;
+            v->data.bytes2 = GETINT16(buf+idx);
+            idx += v->len;
             break;
 
         case szMAPI_INT:
-        case szMAPI_FLOAT:
-        case szMAPI_BOOLEAN:      /* 4 bytes */
+        case szMAPI_FLOAT:      /* 4 bytes */
+        case szMAPI_ERROR:
+        case szMAPI_BOOLEAN:
             a->num_values = 1;
             v = alloc_mapi_values (a);
             v->len = 4;
-            memmove (&v->data, buf+idx, 4);
-            idx += 4;
+            v->data.bytes4 = GETINT32(buf+idx);
+            idx += v->len;
             break;
 
         case szMAPI_DOUBLE:
+        case szMAPI_APPTIME:
+        case szMAPI_CURRENCY:
+        case szMAPI_INT8BYTE:
         case szMAPI_SYSTIME:         /* 8 bytes */
             a->num_values = 1;
             v = alloc_mapi_values (a);
             v->len = 8;
-            memmove (&v->data, buf+idx, 8);
-            idx += 8;
+            v->data.bytes8[0] = GETINT32(buf+idx);
+            v->data.bytes8[1] = GETINT32(buf+idx+4);
+            idx += v->len;
             break;
             
+        case szMAPI_CLSID:
+            a->num_values = 1;
+            v = alloc_mapi_values (a);
+            v->len = sizeof (MAPI_GUID);
+            memmove (&v->data, buf+idx, v->len);
+            idx += v->len;
+            break;
+
         case szMAPI_STRING:
         case szMAPI_UNICODE_STRING:
+        case szMAPI_OBJECT:
         case szMAPI_BINARY:       /* variable length */
+        case szMAPI_UNSPECIFIED:
             {
                 int val_idx = 0;
                 a->num_values = GETINT32(buf+idx); idx += 4;
                 v = alloc_mapi_values (a);
                 for (val_idx = 0; val_idx < a->num_values; val_idx++)
                 {
+                    size_t total_malloc = 0;
+                    
                     v[val_idx].len = GETINT32(buf+idx); idx += 4;
                     /* must pad length to 4 byte boundary */
                     {
@@ -762,8 +860,10 @@ decode_mapi (size_t len, char *buf)
                             v[val_idx].len += (4 - d.rem);
                         }
                     }
-                    v[val_idx].data.buf = MALLOC(v[val_idx].len * 
-                                                 sizeof (char));
+
+                    total_malloc = v[val_idx].len * sizeof (char);
+                    
+                    v[val_idx].data.buf = CHECKED_MALLOC(total_malloc);
                     memmove (v[val_idx].data.buf, 
                              buf+idx, 
                              v[val_idx].len);
@@ -821,13 +921,24 @@ file_add_mapi_attrs (File* file, MAPI_Attr** attrs)
     {
         MAPI_Attr* a = attrs[i];
         
-        switch (a->name)
+        if (a->num_values)
         {
-        case MAPI_ATTACH_LONG_FILENAME:
-            if (file->name) FREE(file->name);
-            file->name = munge_fname (a->values[0].len,
-                                      a->values[0].data.buf);
-            break;
+            
+            switch (a->name)
+            {
+            case MAPI_ATTACH_LONG_FILENAME:
+                if (file->name) FREE(file->name);
+                file->name = munge_fname (a->values[0].len,
+                                          a->values[0].data.buf);
+                break;
+                
+            case MAPI_ATTACH_DATA_OBJ:
+                file->len = a->values[0].len;
+                if (file->data) FREE (file->data);
+                file->data = CHECKED_MALLOC (file->len * sizeof(char));
+                memmove (file->data, a->values[0].data.buf, file->len);
+                break;
+            }
         }
     }
 }
@@ -863,9 +974,7 @@ file_add_attr (File* file, Attr* attr)
         
     case attATTACHDATA:
         file->len = attr->len;
-        file->data = MALLOC(attr->len * sizeof (char));
-        assert (file->data);
-        if (!file->data) abort();
+        file->data = CHECKED_MALLOC(attr->len * sizeof (char));
         memmove (file->data, attr->buf, attr->len);
         break;
     }
@@ -913,8 +1022,7 @@ parse_file (FILE* input_file, char* directory, int flags)
             }
             else
             {
-                file = (File*)MALLOC (sizeof (File));
-                memset (file, '\0', sizeof (File));
+                file = (File*)CHECKED_CALLOC (1, sizeof (File));
             }
         }
         /* Add the data to our lists. */
